#ifndef BWIDGETS_ERROR_HELPER
#define BWIDGETS_ERROR_HELPER

#include <basic_widgets/core/type/exception.hpp>
#include <basic_widgets/core/type/sdl_error.hpp>

namespace bwidgets
{
    template<Exception E, typename T>
    inline auto ptr_or_throw(T* ptr, const std::string& w = "unknown") -> T*
    {
        std::string what;
        if constexpr (std::is_same_v<E, SDLError>)
            what = w == "unknown" ? SDL_GetError() : w;
        else what = w;

        if (ptr == nullptr) throw E(what);
        return ptr;
    }

    template<Exception E, typename T>
    inline auto success_or_throw(
      T code, const std::string& w = "unknown",
      std::function<bool(T)> success = [](T code) { return code == 0; }) -> T
    {
        if constexpr (std::is_same_v<E, SDLError>) {
            std::string what {w == "unknown" ? SDL_GetError() : w};
            if (code < 0) throw E(what);
        }
        else if (success(code)) throw E(w);

        return code;
    }
}

#endif
