#ifndef BWIDGETS_WIDGET_HPP
#define BWIDGETS_WIDGET_HPP

#include <memory>

#include <basic_widgets/core/renderer.hpp>
#include <basic_widgets/core/type/size.hpp>
#include <basic_widgets/w/feat/event_handler_impl.hpp>

union SDL_Event;

struct SDL_Renderer;

namespace bwidgets
{
    class Widget : public virtual EventHandlerImpl
    {
    protected:
        std::shared_ptr<Renderer> _renderer;
        SDL_Rect                  _viewport {0, 0, 0, 0};
        SDL_Rect                  _widget_area {0, 0, 0, 0};

        virtual void _handle_geometry_change(const SDL_Rect&) = 0;
        virtual void _handle_renderer_change(const std::shared_ptr<Renderer>&) {}
        virtual void _handle_rendering() = 0;

    public:
        Widget* parent;

        explicit Widget(Widget* p = nullptr) noexcept : parent {p} {}

        [[nodiscard]] virtual auto size() const noexcept -> Size = 0;

        virtual auto               render() -> Widget* final;
        virtual auto               renderer(std::shared_ptr<Renderer>) -> Widget* final;
        virtual auto               viewport(const SDL_Rect&) -> Widget* final;
        [[nodiscard]] virtual auto viewport() const -> const SDL_Rect& final;
    };
}

#endif
